function applyoptions(md, data, options, canvas){ //{{{
	//APPLYOPTIONS - apply colobar, text, cloud, and expdisp options to current plot
	//
	//   Usage:
	//      applyoptions(md, data, options)
	//
	//   See also: PLOTMODEL, PARSE_OPTIONS

	//{{{ colorbar
	var gl = canvas.gl;
	if (options.exist('colorbar')) {
		if (options.getfieldvalue('colorbar')==1) {
			//{{{ Variable options initialization
			var caxis = options.getfieldvalue('caxis');
			var colorbarinnerlabels = options.getfieldvalue('colorbarinnerlabels','off');
			var ccanvasid, ctitleid, clabelsid, ccanvas, ctitle, clabels, ccontext, cmap, colorbar, cwidth, cheight, cgradient, color, y, x;
			//}}}
			//{{{ Create colorbar labels
			var labels = [];
			var cdivisions = options.getfieldvalue('colorbarnticks', 6);
			var caxisdelta = caxis[1] - caxis[0];
			var precision = options.getfieldvalue('colorbarprecision', 3);
			var format = options.getfieldvalue('colorbarformat', 'f').toLowerCase();
			if (options.getfieldvalue('log','off')!='off') {
				for (var i=cdivisions; i >= 0; i--) {
					var scale = (Math.log10(caxis[1])-Math.log10(caxis[0]))/Math.log10(options.getfieldvalue('log', 10));
					if (format === 'f') {
						labels[i] = (Math.pow(options.getfieldvalue('log', 10), Math.log10(caxis[0])/Math.log10(options.getfieldvalue('log', 10))+scale*(cdivisions-i)/cdivisions)).toFixed(precision);
					}
					else if (format === 'e') {
						labels[i] = (Math.pow(options.getfieldvalue('log', 10), Math.log10(caxis[0])/Math.log10(options.getfieldvalue('log', 10))+scale*(cdivisions-i)/cdivisions)).toPrecision(precision);
					}
					else {
						labels[i] = (Math.pow(options.getfieldvalue('log', 10), Math.log10(caxis[0])/Math.log10(options.getfieldvalue('log', 10))+scale*(cdivisions-i)/cdivisions)).toFixed(precision);
					}
				}
			} else {
				for (var i=cdivisions; i >= 0; i--) {
					if (format === 'f') {
						labels[i] = (caxisdelta*(cdivisions-i)/cdivisions+caxis[0]).toFixed(precision);
					}
					else if (format === 'e') {
						labels[i] = (caxisdelta*(cdivisions-i)/cdivisions+caxis[0]).toPrecision(precision);
					}
					else {
						labels[i] = (caxisdelta*(cdivisions-i)/cdivisions+caxis[0]).toFixed(precision);
					}
				}
			} //}}}
			//{{{ Initialize colorbar canvas
			ccanvasid = options.getfieldvalue('colorbarid', options.getfieldvalue('canvasid').replace('canvas','colorbar-canvas'));
			ccanvas = $('#'+ccanvasid)[0];
			cwidth = ccanvas.width*options.getfieldvalue('colorbarwidth', 1);
			cheight = ccanvas.height*options.getfieldvalue('colorbarheight', 1);
			ccontext = ccanvas.getContext('2d');
			ccontext.clearRect(0, 0, cwidth, cheight);
			ccontext.beginPath();
			cmap = options.getfieldvalue('colormap','jet');
			colorbar = colorbars[cmap];
			cgradient = ccontext.createLinearGradient(0, 0, 0, cheight);
			//}}}
			//{{{ Draw colorbar gradient
			for (var i=0; i < colorbar.length; i++) {
				color = colorbar[colorbar.length-i-1];
				color = [Math.round(color[0]*255), Math.round(color[1]*255), Math.round(color[2]*255)];
				cgradient.addColorStop(i/colorbar.length*(cdivisions/(cdivisions+1.0))+(1.0/(cdivisions+1.0)),'rgba('+color.toString()+', 1.0)');
			}
			ccontext.fillStyle=cgradient;
			ccontext.fillRect(0, 0, cwidth, cheight);
			//}}}
			//{{{ Draw colorbar border
			ccontext.beginPath();
			ccontext.lineWidth='1';
			ccontext.strokeStyle=options.getfieldvalue('colorbarfontcolor','black');
			ccontext.rect(0, 0, cwidth, cheight);
			ccontext.stroke();
			//}}}
			//{{{ Draw colorbar labels
			clabelsid = options.getfieldvalue('colorbarid', ccanvasid).replace('canvas','labels');
			clabels = $('#'+clabelsid);
			if (colorbarinnerlabels=='on') {
				clabels.removeClass('sim-colorbar-labels-outer');
				clabels.addClass('sim-colorbar-labels-inner');
			}
			else {
				clabels.removeClass('sim-colorbar-labels-inner');
				clabels.addClass('sim-colorbar-labels-outer');
			}
			var clabelstring = '';
			clabels.empty();
			for (var i=0; i <= cdivisions; i++) {
				y = (i+0.5)/(cdivisions+1)*cheight;
				x = 0.2*cwidth;
				clabelstring += '<li><span>'+labels[i]+'</span></li>';
				ccontext.beginPath();
				ccontext.moveTo(0, y);
				ccontext.lineTo(x, y);
				ccontext.moveTo(cwidth-x, y);
				ccontext.lineTo(cwidth, y);
				ccontext.stroke();
			}
			clabels.append(clabelstring);
			//}}}
			//{{{ Draw colorbar title
			ctitleid = options.getfieldvalue('colorbarid', ccanvasid).replace('canvas','heading');
			ctitle = $('#'+ctitleid);
			if (options.exist('colorbartitle')) { ctitle.html(options.getfieldvalue('colorbartitle')); }
			//}}}
		}
	} //}}}
	//{{{ texture canvas
	var tcontext, tcanvas, tcanvasid, tURL, tgradient;
	tcanvasid = 'texturecanvas';
	var tcanvas = document.getElementById(tcanvasid);
	if (tcanvas == null) {
		$('<canvas id="texturecanvas" width="256" height="256" style="display: none;"></canvas>').insertAfter('#'+String(options.getfieldvalue('canvasid')));
		tcanvas = document.getElementById(tcanvasid);
	}
	tcontext = tcanvas.getContext('2d');
	tgradient = tcontext.createLinearGradient(0, 0, 0, 256);

	var cmap = options.getfieldvalue('colormap','jet');
	var colorbar = colorbars[cmap];
	for (var i=0; i < colorbar.length; i++) {
		color = colorbar[colorbar.length-i-1];
		color = [Math.round(color[0]*255), Math.round(color[1]*255), Math.round(color[2]*255)];
		tgradient.addColorStop(i/colorbar.length,'rgba('+color.toString()+', 1.0)');
	}

	tcontext.fillStyle = tgradient;
	tcontext.fillRect(0, 0, 256, 256);
	tURL = tcanvas.toDataURL();
	canvas.unitNode.texture = initTexture(canvas.gl, tURL);
	//}}}
	//{{{ text display
	var overlaycanvasid = options.getfieldvalue('overlayid', options.getfieldvalue('canvasid')+'-overlay');
	var overlaycanvas = $('#'+overlaycanvasid)[0];
	if (!isEmptyOrUndefined(overlaycanvas)) {
		//Get drawing context and save reference on main WebGL canvas
		var ctx = overlaycanvas.getContext('2d');
		canvas.overlaycanvas = overlaycanvas;
		
		//Resize interal viewport coordinates to match screenspace coordinates
		var rect = overlaycanvas.getBoundingClientRect();
		overlaycanvas.width  = rect.width;
		overlaycanvas.height = rect.height;
		
		//Clear canvas each frame for any new drawings
		canvas.overlayHandlers['draw'] = function(overlaycanvas) {
			ctx.clearRect(0, 0, overlaycanvas.width, overlaycanvas.height);
		}
	}
	//{{{ lat long overlay
	if (options.exist('latlongoverlay')) {
		var latitudes = {
			//'-90': 1,
			//'-65': .999,
			'-60': 0.994046875,
			'-45': 0.955729166666666,
			'-30': 0.9226562500000024,
			//'-15': 0.830729166666665,
			'0': 0.74218749999999811,
			//'15': 0.63932291666666663,
			'30': 0.523390625000001,
			'45': 0.4020001,
			'60': 0.26953124999999978,
			//'65': 0.225390625,
			//'90': 0.0,
		}
		var longitudes = [-150, -120, -90, -60, -30, 0, 30, 60, 90, 120, 150, 180];
		canvas.overlayHandlers['latlong'] = function(canvas) {
			//Transform from world space to viewport space
			var centerx = overlaycanvas.width / 2;
			var centery = overlaycanvas.height / 2;
			var radius = (overlaycanvas.height) / 2;

			//Draw latitudes
			ctx.setLineDash([5, 10]);
			for(latitude in latitudes) {
				ctx.beginPath();
				ctx.arc(centerx, centery, radius * latitudes[latitude], 0, 2 * Math.PI);
				ctx.stroke();
				ctx.font = 'bold ' + String(options.getfieldvalue('colorbarfontsize', 18))+'px "Lato", Helvetica, Arial, sans-serif';
				ctx.fillStyle = options.getfieldvalue('colorbarfontcolor','black');
				ctx.strokeStyle = options.getfieldvalue('colorbarfontcolor','black');
				ctx.textAlign = 'center';
				ctx.textBaseline = 'middle';
				ctx.fillText(latitude, centerx, centery + radius * latitudes[latitude]);
				ctx.strokeText(latitude, centerx, centery + radius * latitudes[latitude]);
			}
			//Draw longitudes
			ctx.setLineDash([1, 0]);
			for (longitude in longitudes) {
				ctx.beginPath();
				ctx.moveTo(centerx, centery);
				ctx.lineTo(centerx + radius * Math.sin(longitudes[longitude] * DEG2RAD), centery + radius * Math.cos(longitudes[longitude] * DEG2RAD));
				ctx.stroke();
			}
		}
	} //}}}
	if (options.exist('textlabels')) {
		//Attatch new overlay handler to display text labels
		var textLabels = options.getfieldvalue('textlabels',[]);
		canvas.overlayHandlers['text'] = function(canvas) {
			for (var i = 0; i < textLabels.length; i++) {
				//Get text label to display
				var textLabel = textLabels[i];
				textLabel = {
					position: defaultFor(textLabel.position, vec3.create()),
					text: defaultFor(textLabel.text, ''),
					fontSize: defaultFor(textLabel.fontSize, 18),
					fontColor: defaultFor(textLabel.fontColor, 'black'),

				};
				
				// function declared in slr-gfm sim-front-end-controller.js
				// if labels are behind the globe sphere then skip iteartion and do not display them
				if (isLabelVisible(textLabel)) {
					//Transform from world space to viewport space
					var screenPoint = vec3.transformMat4(vec3.create(), textLabel.position, canvas.camera.vpMatrix);
					var x = (screenPoint[0] + 1.0) * (canvas.width / 2) + canvas.selector.offset().left;
					var y = (-screenPoint[1] + 1.0) * (canvas.height / 2) + canvas.selector.offset().top;
					
					//Draw text
					ctx.font = 'bold ' + String(textLabel.fontSize) + 'px Arial Black, sans-serif';
					ctx.fillStyle = textLabel.fontColor;
					ctx.strokeStyle = 'white';
					ctx.textAlign = 'center';
					ctx.textBaseline = 'middle';
					ctx.fillText(textLabel.text, x, y);
					ctx.strokeText(textLabel.text, x, y);
				}
			}
		}
	} //}}}
	//{{{ additional rendering nodes
	if (options.exist('render')) {
		var meshresults = processmesh(md, data, options);
		var x = meshresults[0];
		var y = meshresults[1];
		var z = meshresults[2];
		var elements = meshresults[3];
		var is2d = meshresults[4];
		var isplanet = meshresults[5];

		var xlim = options.getfieldvalue('xlim', [ArrayMin(x), ArrayMax(x)]);
		var ylim = options.getfieldvalue('ylim', [ArrayMin(y), ArrayMax(y)]);
		var zlim = options.getfieldvalue('zlim', [ArrayMin(z), ArrayMax(z)]);

		var global = vec3.length([(xlim[0] + xlim[1]) / 2, (ylim[0] + ylim[1]) / 2, (zlim[0] + zlim[1]) / 2]) < 6371000/10; //tolerance for global models = center is 637100 meters away from center of earth
		var translation = global ? [(xlim[0] + xlim[1]) / 2, (ylim[0] + ylim[1]) / 2, (zlim[0] + zlim[1]) / 2] : canvas.view.position;

		var renderObjects = options.getfieldvalue('render',{});
		for (var renderObject in renderObjects) {
			//Modify renderObejct?
			var object = renderObjects[renderObject];
			object = {
				enabled: defaultFor(object.enabled, true),					//Toggle display of the render object node
				scale: defaultFor(object.scale, 1),							//Model matrix scaling
				x: defaultFor(object.x, [0.0, 1.0, 0.0, 0.0, 0.0, 0.0]),	//x coordinate array
				y: defaultFor(object.y, [0.0, 0.0, 0.0, 1.0, 0.0, 0.0]),	//y coordinate array
				z: defaultFor(object.z, [0.0, 0.0, 0.0, 0.0, 0.0, 1.0]),	//z coordinate array
				indices: defaultFor(object.indices, []),					//indices array
				name: defaultFor(object.name, 'NY'),						//Text to display for cities.
				size: defaultFor(object.size, 1),							//Physical size of the object in meters
				color: defaultFor(object.color, 'black'),					//Diffuse color of object
				height: defaultFor(object.height, 25000),					//Height of object along y axis, currently for clouds only
				range: defaultFor(object.range, 120000),						//Range of sz plane to spawn object, currently for clouds only
				quantity: defaultFor(object.quantity, 15)					//Quantity of objects to display, currently for clouds only
			};
			if (!object.enabled) { continue; }
			if ('sky' === renderObject && !('sky' in canvas.nodes)) {
				var mesh = GL.Mesh.icosahedron({size: 6371000 * canvas.atmosphere.scaleHeight, subdivisions: 5});
				var texture = initTexture(gl, canvas.rootPath + 'textures/TychoSkymapII_t4_2k.jpg');
				node = new Node(
					'canvas', canvas,
					'options', options,
					'renderObject', object,
					'name', 'sky',
					'shaderName', 'SkyFromSpace',
					'cullFace', gl.FRONT,
					'mesh', mesh,
					'texture',texture,
					'translation',translation
				);
			}
			if ('space' === renderObject && !('space' in canvas.nodes)) {
				var mesh = GL.Mesh.sphere({size: 6371000 * 20});
				var texture = initTexture(gl, canvas.rootPath + 'textures/TychoSkymapII_t4_2k.jpg');
				node = new Node(
					'canvas', canvas,
					'options', options,
					'renderObject', object,
					'name', 'space',
					'shaderName', 'Textured',
					'cullFace', gl.FRONT,
					'drawOrder', 2,
					'mesh', mesh,
					'texture',texture,
					'translation',translation
				);
			}
			if ('coastlines' === renderObject && !('coastlines' in canvas.nodes)) {
				node = new Node(
					'canvas', canvas,
					'options', options,
					'renderObject', object,
					'name', 'coastlines',
					'shaderName', 'Colored',
					'drawMode', gl.LINE_STRIP,
					'lineWidth', options.getfieldvalue('linewidth', 1),
					'scale', [object.scale, object.scale, object.scale],
					'rotation', [-90, 0, 0]
				);
				node.patch('Vertices', [object.x, object.y, object.z], 'FaceColor', 'none');
			}
			if ('graticule' === renderObject && !('graticule' in canvas.nodes)) {
				node = new Node(
					'canvas', canvas,
					'options', options,
					'renderObject', object,
					'name', 'graticule',
					'shaderName', 'Colored',
					'drawMode', gl.LINE_STRIP,
					'lineWidth', options.getfieldvalue('linewidth', 1),
					'scale', [object.scale, object.scale, object.scale],
					'rotation', [-90, 180, 0]
				);
				node.patch('Vertices', [object.x, object.y, object.z], 'FaceColor', 'none');
			}
			if ('cities' === renderObject && !('cities' in canvas.nodes)) {
				var mesh = GL.Mesh.icosahedron({size: object.size, subdivisions: 1});
				node = new Node(
					'canvas', canvas,
					'options', options,
					'renderObject', object,
					'name', 'cities',
					'shaderName', 'ColoredDiffuse',
					'diffuseColor', object.color,
					'lineWidth', options.getfieldvalue('linewidth', 1),
					'scale', [object.scale, object.scale, object.scale],
					'rotation', [-90, 0, 0]
				);
				node.geometryShader('Mesh', mesh, 'Vertices', [object.x, object.y, object.z], 'Indices', object.indices);
			}
			if ('axis' === renderObject && !('axis' in canvas.nodes)) {		
				node = new Node(
					'canvas', canvas,
					'options', options,
					'renderObject', object,
					'name', 'axis',
					'shaderName', 'Colored',
					'drawMode', gl.LINES,
					'lineWidth', options.getfieldvalue('linewidth', 1),
					'scale', [object.scale, object.scale, object.scale],
					'rotation', [0, 0, 0]
				);
				node.patch('Vertices', [object.x, object.y, object.z], 'FaceColor', 'none');
			}
			if ('city' === renderObject) {
				//city
				var mesh = GL.Mesh.sphere({size: object.size});
				node = new Node(
					'canvas', canvas,
					'options', options,
					'renderObject', object,
					'name', 'city',
					'shaderName', 'ColoredDiffuse',
					'diffuseColor', object.color,
					'mesh', mesh,
					'translation', [object.x, object.z, -object.y]
				);
				//Attatch new overlay handler to display city name
				canvas.overlayHandlers['city'] = function(canvas) {
					var node = canvas.nodes['city'];
					var object = node.renderObject;
					var screenPoint = vec3.transformMat4(vec3.create(), node.translation, canvas.camera.vpMatrix);
					var x = (screenPoint[0] + 1.0) * (canvas.width / 2) + canvas.selector.offset().left;
					var y = (-screenPoint[1] + 1.0) * (canvas.height / 2) + canvas.selector.offset().top;

					ctx.font = 'bold ' + String(options.getfieldvalue('colorbarfontsize', 22))+'px Arial Black, sans-serif';
					ctx.fillStyle = options.getfieldvalue('colorbarfontcolor','black');
					ctx.strokeStyle = 'white';
					ctx.textAlign = 'center';
					ctx.textBaseline = 'middle';
					ctx.fillText(object.name, x, y);
					ctx.strokeText(object.name, x, y);
				}
			}
			if ('clouds' === renderObject && !('clouds0' in canvas.nodes)) {
				//clouds				
				var mesh = GL.Mesh.fromURL(canvas.rootPath+'obj/cloud.obj');
				for (var i = 0; i < object.quantity; i++) {
					//TODO: More options, less magic numbers. Add animation. Better shading.
					var offset = [	translation[0] + (Math.random() - 0.5) * 2 * object.range, 
									translation[1] + object.height + (Math.random() - 0.5) * 0.2 * object.range, 
									translation[2] + (Math.random() - 0.5) * 2 * object.range];
					node = new Node(
						'canvas', canvas,
						'options', options,
						'renderObject', object,
						'name', 'clouds' + i,
						'shaderName', 'ColoredDiffuse',
						'animation', {'time': Date.now(),'target': translation,'current': translation},
						'diffuseColor', [0.7,0.7,0.7,1.0],
						'specularColor', [0.0,0.0,0.0,1.0],
						'mesh', mesh,
						'scale', [object.scale, object.scale, object.scale],
						'translation', offset
					);
				}
			}
		}
	} //}}}
} //}}}
